from django import forms


class AppForm(forms.Form):
    name = forms.CharField(max_length=60)
    platform = forms.ChoiceField()
    plan = forms.ChoiceField(required=False)
    teamOwner = forms.ChoiceField(required=False)
    pool = forms.ChoiceField(required=False)


class AppAddTeamForm(forms.Form):
    team = forms.CharField(max_length=60)


class RunForm(forms.Form):
    app = forms.CharField(max_length=60)
    command = forms.CharField(max_length=1000)


class SetEnvForm(forms.Form):
    env = forms.CharField(max_length=1000)



import re

from django import forms
from django.core import validators

team_name = re.compile(r"^[a-zA-Z][-@_.+\w\s]+$")
invalid_team_msg = ("Team name must start with a letter and "
                    "have at least two characters.")
team_name_validator = validators.RegexValidator(regex=team_name,
                                                message=invalid_team_msg)


class TeamForm(forms.Form):
    name = forms.CharField(max_length=60, validators=[team_name_validator])



from tsuru_dashboard import settings

import requests
import json


class MetricNotEnabled(Exception):
    pass


def metrics_from_api(app):
    for unit in app.get("units", []):
        if "ProcessName" in unit:
            return True
    return False


def get_backend(app, token):
    if metrics_from_api(app):
        headers = {'authorization': token}
        url = "{}/apps/{}/metric/envs".format(settings.TSURU_HOST, app["name"])
        response = requests.get(url, headers=headers)

        if response.status_code == 200:
            data = response.json()
            if "METRICS_ELASTICSEARCH_HOST" in data:
                return ElasticSearch(data["METRICS_ELASTICSEARCH_HOST"], ".measure-tsuru-*", app["name"])
    else:
        if "envs" in app and "ELASTICSEARCH_HOST" in app["envs"]:
            return ElasticSearch(app["envs"]["ELASTICSEARCH_HOST"], ".measure-tsuru-*", app["name"])

    raise MetricNotEnabled


class ElasticSearch(object):
    def __init__(self, url, index, app, process_name=None):
        self.index = index
        self.app = app
        self.url = url

    def post(self, data, metric):
        url = "{}/.measure-tsuru-*/{}/_search".format(self.url, metric)
        result = requests.post(url, data=json.dumps(data))
        return result.json()

    def process(self, data, formatter=None):
        if not formatter:
            def default_formatter(x):
                return x
            formatter = default_formatter

        result = {}
        min_value = None
        max_value = 0

        if "aggregations" in data and data["aggregations"]["range"]["buckets"][0]["doc_count"] > 0:
            for bucket in data["aggregations"]["range"]["buckets"][0]["date"]["buckets"]:
                bucket_max = formatter(bucket["max"]["value"])
                bucket_min = formatter(bucket["min"]["value"])
                bucket_avg = formatter(bucket["avg"]["value"])

                if min_value is None:
                    min_value = bucket_min

                if bucket_min < min_value:
                    min_value = bucket_min

                if bucket_max > max_value:
                    max_value = bucket_max

                if not result:
                    result = {
                        "max": [],
                        "min": [],
                        "avg": [],
                    }
                result["max"].append([bucket["key"], "{0:.2f}".format(bucket_max)])
                result["min"].append([bucket["key"], "{0:.2f}".format(bucket_min)])
                result["avg"].append([bucket["key"], "{0:.2f}".format(bucket_avg)])

        return {
            "data": result,
            "min": "{0:.2f}".format(min_value or 0),
            "max": "{0:.2f}".format(max_value + 1),
        }

    def cpu_max(self, date_range=None, interval=None, process_name=None):
        query = self.query(date_range=date_range, interval=interval, process_name=process_name)
        response = self.post(query, "cpu_max")
        process = self.process(response)
        return process

    def mem_max(self, date_range=None, interval=None, process_name=None):
        query = self.query(date_range=date_range, interval=interval, process_name=process_name)
        return self.process(self.post(query, "mem_max"), formatter=lambda x: x / (1024 * 1024))

    def units(self, date_range=None, interval=None, process_name=None):
        aggregation = {"units": {"cardinality": {"field": "host"}}}
        query = self.query(date_range=date_range, interval=interval,
                           process_name=process_name, aggregation=aggregation)
        return self.units_process(self.post(query, "cpu_max"))

    def units_process(self, data, formatter=None):
        result = {}
        min_value = None
        max_value = 0

        if "aggregations" in data and data["aggregations"]["range"]["buckets"][0]["doc_count"] > 0:
            for bucket in data["aggregations"]["range"]["buckets"][0]["date"]["buckets"]:
                value = bucket["units"]["value"]

                if min_value is None:
                    min_value = value

                if value < min_value:
                    min_value = value

                if value > max_value:
                    max_value = value

                if not result:
                    result["units"] = []

                result["units"].append([bucket["key"], "{0:.2f}".format(value)])

        return {
            "data": result,
            "min": "{0:.2f}".format(min_value or 0),
            "max": "{0:.2f}".format(max_value),
        }

    def requests_min(self, date_range=None, interval=None, process_name=None):
        aggregation = {"sum": {"sum": {"field": "count"}}}
        query = self.query(date_range=date_range, interval=interval,
                           process_name=process_name, aggregation=aggregation)
        return self.requests_min_process(self.post(query, "response_time"))

    def requests_min_process(self, data, formatter=None):
        result = {}
        min_value = None
        max_value = 0

        if "aggregations" in data and data["aggregations"]["range"]["buckets"][0]["doc_count"] > 0:
            for bucket in data["aggregations"]["range"]["buckets"][0]["date"]["buckets"]:
                value = bucket["sum"]["value"]

                if min_value is None:
                    min_value = value

                if value < min_value:
                    min_value = value

                if value > max_value:
                    max_value = value

                if not result:
                    result["requests"] = []
                result["requests"].append([bucket["key"], value])

        return {
            "data": result,
            "min": min_value,
            "max": max_value,
        }

    def response_time(self, date_range=None, interval=None, process_name=None):
        query = self.query(date_range=date_range, interval=interval, process_name=process_name)
        return self.process(self.post(query, "response_time"))

    def connections(self, date_range=None, interval=None, process_name=None):
        aggregation = {"connection": {"terms": {"field": "connection.raw"}}}
        query = self.query(date_range=date_range, interval=interval,
                           process_name=process_name, aggregation=aggregation)
        return self.connections_process(self.post(query, "connection"))

    def connections_process(self, data, formatter=None):
        result = {}
        min_value = 0
        max_value = 0

        if "aggregations" in data and data["aggregations"]["range"]["buckets"][0]["doc_count"] > 0:
            for bucket in data["aggregations"]["range"]["buckets"][0]["date"]["buckets"]:
                for doc in bucket["connection"]["buckets"]:
                    size = doc["doc_count"]
                    conn = doc["key"]
                    if conn not in result:
                        result[conn] = []

                    if size < min_value:
                        min_value = size

                    if size > max_value:
                        max_value = size

                    result[conn].append([bucket["key"], size])

        return {
            "data": result,
            "min": min_value,
            "max": max_value,
        }

    def query(self, date_range=None, interval=None, aggregation=None, process_name=None):
        if not date_range:
            date_range = "1h"

        if not interval:
            interval = "1m"

        f = {
            "bool": {
                "must": [{
                    "term": {
                        "app": self.app,
                        "app.raw": self.app,
                    }
                }],
            }
        }

        if process_name:
            p = {
                "term": {
                    "process.raw": process_name,
                    "process": process_name,
                },
            }
            f["bool"]["must"].append(p)

        query_filter = {
            "filtered": {
                "filter": f
            }
        }
        if not aggregation:
            aggregation = {
                "max": {"max": {"field": "value"}},
                "min": {"min": {"field": "value"}},
                "avg": {"avg": {"field": "value"}}
            }
        return {
            "query": query_filter,
            "size": 0,
            "aggs": {
                "range": {
                    "date_range": {
                        "field": "@timestamp",
                        "ranges": [{
                            "from": "now-" + date_range,
                            "to": "now"
                        }]
                    },
                    "aggs": {
                        "date": {
                            "date_histogram": {
                                "field": "@timestamp",
                                "interval": interval
                            },
                            "aggs": aggregation,
                        }
                    }
                }
            }
        }



import json
import tarfile
import datetime
import calendar
from cStringIO import StringIO
from zipfile import ZipFile
from base64 import decodestring

import requests

from django.template.response import TemplateResponse
from django.http import HttpResponse, HttpResponseServerError, Http404, StreamingHttpResponse, JsonResponse
from django.shortcuts import redirect
from django.core.urlresolvers import reverse
from django.views.generic import TemplateView
from django.contrib import messages

from pygments import highlight
from pygments.lexers import DiffLexer
from pygments.formatters import HtmlFormatter

from tsuru_dashboard import settings
from tsuru_dashboard.auth.views import LoginRequiredView, LoginRequiredMixin

from .forms import AppForm, AppAddTeamForm, RunForm, SetEnvForm


class DeployInfo(LoginRequiredMixin, TemplateView):
    template_name = 'apps/deploy.html'

    def get_app(self, app_name):
        url = '{}/apps/{}'.format(settings.TSURU_HOST, app_name)
        return requests.get(url, headers=self.authorization).json()

    def get_context_data(self, *args, **kwargs):
        deploy_id = kwargs['deploy']
        context = super(DeployInfo, self).get_context_data(*args, **kwargs)

        url = '{}/deploys/{}'.format(settings.TSURU_HOST, deploy_id)
        response = requests.get(url, headers=self.authorization)
        context['deploy'] = response.json()

        diff = context['deploy'].get('Diff')
        if diff and diff != u'The deployment must have at least two commits for the diff.':
            format = HtmlFormatter()
            diff = highlight(diff, DiffLexer(), format)
        else:
            diff = None

        context['deploy']['Diff'] = diff

        app_name = kwargs['app_name']
        context['app'] = self.get_app(app_name)
        return context


class ListDeploy(LoginRequiredView):
    template = 'apps/deploys.html'

    def zip_to_targz(self, zip_file):
        fd = StringIO()
        tar = tarfile.open(fileobj=fd, mode='w:gz')
        timeshift = int((datetime.datetime.now() - datetime.datetime.utcnow()).total_seconds())

        with ZipFile(zip_file) as f:
            for zip_info in f.infolist():
                tar_info = tarfile.TarInfo(name=zip_info.filename)
                tar_info.size = zip_info.file_size
                tar_info.mtime = calendar.timegm(zip_info.date_time) - timeshift

                if zip_info.filename.endswith("/"):
                    tar_info.mode = 0755
                    tar_info.type = tarfile.DIRTYPE
                else:
                    tar_info.mode = 0644
                    tar_info.type = tarfile.REGTYPE

                tar.addfile(tar_info, f.open(zip_info.filename))

        tar.close()
        fd.seek(0)
        return fd

    def read_zip(self, request):
        fd = StringIO()
        fd.write(decodestring(request.POST['filecontent']))
        fd.seek(0)
        return fd

    def deploy(self, request, app_name, tar_file):
        def sending_stream():
            origin = 'drag-and-drop'
            url = '{}/apps/{}/deploy?origin={}'.format(settings.TSURU_HOST, app_name, origin)
            r = requests.post(url, headers=self.authorization, files={'file': tar_file}, stream=True)
            for line in r.iter_lines():
                yield "{}<br>".format(line)
        return StreamingHttpResponse(sending_stream())

    def post(self, request, *args, **kwargs):
        app_name = kwargs['app_name']
        zip_file = self.read_zip(request)
        tar_file = self.zip_to_targz(zip_file)
        return self.deploy(request, app_name, tar_file)

    def get_app(self, app_name):
        url = '{}/apps/{}'.format(settings.TSURU_HOST, app_name)
        return requests.get(url, headers=self.authorization).json()

    def get(self, request, *args, **kwargs):
        app_name = kwargs['app_name']

        page = int(request.GET.get('page', '1'))

        skip = (page * 20) - 20
        limit = page * 20

        url = '{}/deploys?app={}&skip={}&limit={}'.format(
            settings.TSURU_HOST, app_name, skip, limit)
        response = requests.get(url, headers=self.authorization)

        deploys = []
        if response.status_code != 204:
            deploys = response.json() or []

        context = {}
        context['deploys'] = deploys
        context['app'] = self.get_app(app_name)

        if len(deploys) >= 20:
            context['next'] = page + 1

        if page > 0:
            context['previous'] = page - 1

        return TemplateResponse(request, self.template, context=context)


class ChangeUnit(LoginRequiredView):
    def add_unit(self, units, app_name):
        requests.put(
            '{}/apps/{}/units'.format(settings.TSURU_HOST, app_name),
            headers=self.authorization,
            data=str(units)
        )

    def remove_units(self, units, app_name):
        requests.delete(
            '{}/apps/{}/units'.format(settings.TSURU_HOST, app_name),
            headers=self.authorization,
            data=str(units)
        )

    def get_app(self, app_name):
        url = '{}/apps/{}'.format(settings.TSURU_HOST, app_name)
        return requests.get(url, headers=self.authorization).json()

    def post(self, request, *args, **kwargs):
        app_name = kwargs['app_name']

        app = self.get_app(app_name)

        app_units = len(app['units'])
        units = int(request.POST['units'])

        if len(app['units']) < int(request.POST['units']):
            self.add_unit(units - app_units, app_name)

        if len(app['units']) > int(request.POST['units']):
            self.remove_units(app_units - units, app_name)

        return redirect(reverse('detail-app', args=[app_name]))


class AppDetail(LoginRequiredMixin, TemplateView):
    template_name = 'apps/details.html'

    def service_instances(self, app_name):
        tsuru_url = '{}/services/instances?app={}'.format(settings.TSURU_HOST, app_name)
        return requests.get(tsuru_url, headers=self.authorization).json()

    def get_context_data(self, *args, **kwargs):
        context = super(AppDetail, self).get_context_data(*args, **kwargs)
        app_name = kwargs['app_name']
        token = self.request.session.get('tsuru_token')
        url = '{}/apps/{}'.format(settings.TSURU_HOST, app_name)
        headers = {
            'content-type': 'application/json',
            'Authorization': token,
        }

        response = requests.get(url, headers=headers)
        if response.status_code == 404:
            raise Http404()

        context['app'] = response.json()

        service_instances = []

        for service in self.service_instances(app_name):
            if service['instances']:
                service_instances.append(
                    {'name': service['instances'][0], 'servicename': service['service']}
                )

        context['app']['service_instances'] = service_instances
        return context


class CreateApp(LoginRequiredView):
    template_name = 'apps/create.html'

    def render(self, request, context):
        return TemplateResponse(request, self.template_name, context)

    def get(self, request):
        form = AppForm()
        default, plans = self.plans(request)
        form.fields['plan'].choices = plans
        form.fields['plan'].initial = default
        form.fields['platform'].choices = self.platforms(request)
        form.fields['teamOwner'].choices = self.teams(request)
        form.fields['pool'].choices = self.pools(request)
        context = {
            'app_form': form,
        }
        return self.render(request, context)

    def post(self, request):
        context = {}
        form = AppForm(request.POST)
        default, plans = self.plans(request)
        form.fields['plan'].choices = plans
        form.fields['platform'].choices = self.platforms(request)
        form.fields['teamOwner'].choices = self.teams(request)
        form.fields['pool'].choices = self.pools(request)
        if form.is_valid():
            authorization = {'authorization': request.session.get('tsuru_token')}

            # removing keys with empty values
            data = {key: value for key, value in form.cleaned_data.items() if value}

            if 'plan' in data:
                data['plan'] = {'name': data['plan']}

            data = json.dumps(data)

            url = '{}/apps'.format(settings.TSURU_HOST)
            response = requests.post(url, data=data, headers=authorization)

            if response.status_code == 200:
                messages.success(request, u'App was successfully created', fail_silently=True)
                return redirect(reverse('list-app'))

            messages.error(request, response.content, fail_silently=True)

        form.fields['plan'].initial = default
        context['app_form'] = form
        return self.render(request, context)

    def pools(self, request):
        authorization = {'authorization': request.session.get('tsuru_token')}
        url = '{}/pools'.format(settings.TSURU_HOST)
        response = requests.get(url, headers=authorization)
        pools = set()
        pools_json = response.json()
        pools_by_team = pools_json

        # backward compatibility
        if isinstance(pools_json, dict):
            pools_by_team = pools_json["pools_by_team"]
            for pool in pools_json.get('public_pools', []):
                pools.add(pool.get("Name", pool))

        for team_list in pools_by_team:
            for pool in team_list['Pools']:
                pools.add(pool)
        result = [('', '')]
        result.extend([(p, p) for p in pools])
        return result

    def teams(self, request):
        authorization = {'authorization': request.session.get('tsuru_token')}
        url = '{}/teams'.format(settings.TSURU_HOST)
        result = [("", "")]
        response = requests.get(url, headers=authorization)
        if response.status_code != 204:
            teams = response.json()
            result.extend([(t['name'], t['name']) for t in teams])
        return result

    def platforms(self, request):
        authorization = {'authorization': request.session.get('tsuru_token')}
        response = requests.get('{}/platforms'.format(settings.TSURU_HOST), headers=authorization)
        platforms = response.json()
        result = [(', ')]
        result.extend([(p['Name'], p['Name']) for p in platforms])
        return result

    def plans(self, request):
        authorization = {'authorization': request.session.get('tsuru_token')}
        url = '{}/plans'.format(settings.TSURU_HOST)
        response = requests.get(url, headers=authorization)
        plans = response.json() or []
        plan_list = [('', '')]
        default = ''
        for p in plans:
            if p.get('default'):
                default = p['name']
            plan_list.append((p['name'], p['name']))
        return default, plan_list


class AppAddTeam(LoginRequiredView):
    template = 'apps/app_add_team.html'

    def get(self, request, app_name):
        context = {}
        context['app_name'] = app_name
        context['app'] = {'name': app_name}
        context['form'] = AppAddTeamForm()
        context['teams'] = self._get_teams(request)
        return TemplateResponse(request, self.template, context=context)

    def post(self, request, app_name):
        form = AppAddTeamForm(request.POST)
        if not form.is_valid():
            return TemplateResponse(request, self.template, {'form': form})

        authorization = {'authorization': request.session.get('tsuru_token')}
        tsuru_url = '{}/apps/{}/{}'.format(
            settings.TSURU_HOST, app_name, form.data.get('team'))
        response = requests.put(tsuru_url, headers=authorization)
        if response.status_code == 200:
            return TemplateResponse(
                request, self.template,
                {'form': form, 'app_name': app_name,
                 'message': 'The Team was successfully added'}
            )
        return TemplateResponse(request, self.template,
                                {'form': form, 'app_name': app_name,
                                 'errors': response.content})

    def _get_teams(self, request):
        authorization = {'authorization': request.session.get('tsuru_token')}
        response = requests.get('{}/teams'.format(settings.TSURU_HOST),
                                headers=authorization)
        teams = response.json()
        return [t['name'] for t in teams]


class AppRevokeTeam(LoginRequiredView):
    def get(self, request, app_name, team):
        app_name = self.kwargs['app_name']

        authorization = {'authorization': request.session.get('tsuru_token')}
        tsuru_url = '{}/apps/{}/{}'.format(
            settings.TSURU_HOST, app_name, team)
        requests.delete(tsuru_url, headers=authorization)

        return redirect(reverse('app-teams', args=[app_name]))


class RemoveApp(LoginRequiredView):
    def get(self, request, *args, **kwargs):
        app_name = self.kwargs['name']
        authorization = {'authorization': request.session.get('tsuru_token')}
        response = requests.delete(
            '{}/apps/{}'.format(settings.TSURU_HOST, app_name),
            headers=authorization
        )
        if response.status_code > 399:
            return HttpResponse(response.text, status=response.status_code)
        return redirect(reverse('list-app'))


class ListApp(LoginRequiredMixin, TemplateView):
    template_name = "apps/list.html"


class ListAppJson(LoginRequiredView):
    def list_apps(self, name=None):
        url = "{}/apps".format(settings.TSURU_HOST)

        if name:
            url = "{}?name={}".format(url, name)

        response = requests.get(url, headers=self.authorization)

        apps = []
        if response.status_code != 204:
            apps = sorted(response.json(), key=lambda item: item['name'])

        return apps

    def get(self, *args, **kwargs):
        app_list = {"apps": self.list_apps(self.request.GET.get("name"))}
        return JsonResponse(app_list, safe=False)


class AppDetailJson(LoginRequiredView):

    def get_containers(self, app_name):
        url = '{}/docker/node/apps/{}/containers'.format(settings.TSURU_HOST, app_name)
        response = requests.get(url, headers=self.authorization)

        if response.status_code != 200:
            return []

        data = response.json()
        if not data:
            return []

        return data

    def get(self, *args, **kwargs):
        context = {}
        app_name = kwargs['app_name']
        token = self.request.session.get('tsuru_token')
        url = '{}/apps/{}'.format(settings.TSURU_HOST, app_name)
        headers = {
            'content-type': 'application/json',
            'Authorization': token,
        }

        response = requests.get(url, headers=headers)
        if response.status_code == 404:
            raise Http404()

        context['app'] = response.json()

        units_by_status = {}
        for unit in context['app']['units']:
            if unit['Status'] not in units_by_status:
                units_by_status[unit['Status']] = [unit]
            else:
                units_by_status[unit['Status']].append(unit)

        for container in self.get_containers(app_name):
            for index, unit in enumerate(context['app']['units']):
                if self.id_or_name(unit) == container['ID']:
                    context['app']['units'][index].update({
                        'HostAddr': container['HostAddr'],
                        'HostPort': container['HostPort'],
                    })
        context['units_by_status'] = units_by_status
        context['process_list'] = self.process_list(context['app'])
        return JsonResponse(context, safe=False)

    def process_list(self, app):
        process = set()

        for unit in app.get('units', []):
            if 'ProcessName' in unit:
                process.add(unit['ProcessName'])

        return list(process)

    def id_or_name(self, unit):
        if "ID" in unit:
            return unit["ID"]
        return unit["Name"]


class Run(LoginRequiredView):
    template = 'apps/run.html'

    def get(self, request):
        context = {}
        context['form'] = RunForm()
        context['apps'] = self._get_apps(request)
        return TemplateResponse(request, self.template, context=context)

    def post(self, request):
        form = RunForm(request.POST)
        if not form.is_valid():
            return TemplateResponse(request, self.template, {'form': form})

        authorization = {'authorization': request.session.get('tsuru_token')}
        tsuru_url = '{}/apps/{}/run'.format(
            settings.TSURU_HOST, form.data.get('app'))
        response = requests.post(tsuru_url, data=form.data.get('command'),
                                 headers=authorization)
        if response.status_code == 200:
            return TemplateResponse(request, self.template,
                                    {'form': form,
                                     'message': response.content})
        return TemplateResponse(request, self.template,
                                {'form': form, 'errors': response.content})

    def _get_apps(self, request):
        authorization = {'authorization': request.session.get('tsuru_token')}
        response = requests.get('{}/apps'.format(settings.TSURU_HOST),
                                headers=authorization)
        apps = response.json()
        return [a['name'] for a in apps]


class LogStream(LoginRequiredView):
    def get(self, request, *args, **kwargs):
        app_name = kwargs['app_name']

        def sending_stream():
            url = '{}/apps/{}/log?lines=15&follow=1'.format(settings.TSURU_HOST, app_name)
            r = requests.get(url, headers=self.authorization, stream=True)
            for line in r.iter_lines():
                yield line

        return StreamingHttpResponse(sending_stream())


class AppLog(LoginRequiredMixin, TemplateView):
    template_name = 'apps/app_log.html'

    def get_context_data(self, *args, **kwargs):
        context = super(AppLog, self).get_context_data(*args, **kwargs)
        app_name = kwargs['app_name']
        app_url = '{}/apps/{}'.format(settings.TSURU_HOST, app_name)
        app = requests.get(app_url, headers=self.authorization).json()
        context['app'] = app
        return context


class AppTeams(LoginRequiredMixin, TemplateView):
    template_name = "apps/app_team.html"

    def get_context_data(self, *args, **kwargs):
        context = super(AppTeams, self).get_context_data(*args, **kwargs)

        url = '{}/apps/{}'.format(settings.TSURU_HOST, kwargs["app_name"])
        response = requests.get(url, headers=self.authorization)

        if response.status_code == 200:
            app = response.json()
            context.update({"app": app})
        else:
            context.update({"errors": response.content})

        return context


class AppEnv(LoginRequiredView):
    template = 'apps/app_env.html'

    def get(self, request, app_name):
        context = {}
        context['app'] = app_name
        context['form'] = SetEnvForm(initial=context)

        response = self.get_envs(request, app_name)

        if response.status_code == 200:
            envs = response.content.split('\n')
            context['envs'] = envs
            return TemplateResponse(request, self.template, context)
        return TemplateResponse(request, self.template,
                                {'errors': response.content})

    def post(self, request, app_name):
        context = {}
        context['app'] = app_name

        response = self.get_envs(request, app_name)
        if response.status_code == 200:
            form = SetEnvForm(request.POST)
            context['form'] = form
            if not form.is_valid():
                return TemplateResponse(request, self.template, context)

            envs = response.content.split('\n')
            envs.append(request.POST['env'])
            context['envs'] = envs

            response = self.set_env(request, app_name, form)

            if response.status_code == 200:
                context['message'] = response.content
                return TemplateResponse(request, self.template, context)

        context['errors'] = response.content
        return TemplateResponse(request, self.template, context)

    def get_envs(self, request, app_name):
        authorization = {'authorization': request.session.get('tsuru_token')}
        tsuru_url = '{}/apps/{}/env'.format(settings.TSURU_HOST, app_name)
        return requests.get(tsuru_url, headers=authorization)

    def set_env(self, request, app_name, form):
        authorization = {'authorization': request.session.get('tsuru_token')}
        tsuru_url = '{}/apps/{}/env'.format(settings.TSURU_HOST, app_name)
        return requests.post(tsuru_url, data=form.data.get('env'),
                             headers=authorization)


class MetricDetail(LoginRequiredMixin, TemplateView):
    template_name = 'apps/metric_details.html'

    def get_envs(self, app_name):
        url = '{}/apps/{}/env'.format(settings.TSURU_HOST, app_name)
        return requests.get(url, headers=self.authorization).json()

    def get_app(self, app_name):
        url = '{}/apps/{}'.format(settings.TSURU_HOST, app_name)
        return requests.get(url, headers=self.authorization).json()

    def get_context_data(self, *args, **kwargs):
        context = super(MetricDetail, self).get_context_data(*args, **kwargs)
        app_name = kwargs['app_name']

        context['app'] = self.get_app(app_name)
        context['app']['envs'] = self.get_envs(app_name)

        return context


class AppRollback(LoginRequiredView):
    def get(self, request, app_name, image):
        origin = "rollback"
        url = '{}/apps/{}/deploy/rollback?origin={}'.format(settings.TSURU_HOST, app_name, origin)
        response = requests.post(url, headers=self.authorization, data={'image': image})
        if response.status_code == 200:
            return redirect(reverse('app-deploys', args=[app_name]))
        return HttpResponseServerError('NOT OK')


class Settings(LoginRequiredMixin, TemplateView):
    template_name = 'apps/settings.html'

    def get_envs(self, app_name):
        url = '{}/apps/{}/env'.format(settings.TSURU_HOST, app_name)
        return requests.get(url, headers=self.authorization).json()

    def get_context_data(self, *args, **kwargs):
        context = super(Settings, self).get_context_data(*args, **kwargs)
        app_name = kwargs['app_name']
        token = self.request.session.get('tsuru_token')
        url = '{}/apps/{}'.format(settings.TSURU_HOST, app_name)
        headers = {
            'content-type': 'application/json',
            'Authorization': token,
        }

        context['app'] = requests.get(url, headers=headers).json()
        context['app']['envs'] = self.get_envs(app_name)

        return context


class Unlock(LoginRequiredView):
    def get(self, request, *args, **kwargs):
        app_name = self.kwargs['name']
        response = requests.delete(
            '{}/apps/{}/lock'.format(settings.TSURU_HOST, app_name),
            headers=self.authorization
        )

        if response.status_code > 399:
            messages.error(request, response.text, fail_silently=True)
        else:
            messages.success(request, u'App was successfully unlocked', fail_silently=True)

        return redirect(reverse('app-settings', args=[app_name]))



from django.test import TestCase
from django.forms import CharField

from tsuru_dashboard.apps import forms


class SetEnvFormTest(TestCase):
    def test_forms_should_contains_SetEnvForm(self):
        self.assertTrue(hasattr(forms, 'SetEnvForm'))

    def test_should_have_env_field(self):
        self.assertIn('env', forms.SetEnvForm.base_fields)

    def test_env_field_should_have_CharField(self):
        field = forms.SetEnvForm.base_fields['env']
        self.assertIsInstance(field, CharField)

    def test_env_field_should_have_at_most_1000_characters(self):
        field = forms.SetEnvForm.base_fields['env']
        self.assertEqual(1000, field.max_length)



from django.http import HttpResponse, HttpResponseBadRequest

from tsuru_dashboard import settings
from tsuru_dashboard.auth.views import LoginRequiredView

import json
import requests


class Metric(LoginRequiredView):
    def get_app(self, app_name):
        url = '{}/apps/{}'.format(settings.TSURU_HOST, app_name)
        return requests.get(url, headers=self.authorization).json()

    def get_envs(self, request, app_name):
        url = '{}/apps/{}/env'.format(settings.TSURU_HOST, app_name)
        data = requests.get(url, headers=self.authorization).json()
        envs = {}

        for env in data:
            envs[env['name']] = env['value']

        return envs

    def get(self, *args, **kwargs):
        metric = self.request.GET.get("metric")
        if not metric:
            return HttpResponseBadRequest()

        process_name = self.request.GET.get("process_name")

        app_name = kwargs['app_name']
        app = self.get_app(app_name)
        app["envs"] = self.get_envs(self.request, app_name)

        from .backend import get_backend
        token = self.request.session.get('tsuru_token')
        backend = get_backend(app, token)

        interval = self.request.GET.get("interval")
        date_range = self.request.GET.get("date_range")
        data = getattr(backend, metric)(date_range=date_range, interval=interval, process_name=process_name)
        return HttpResponse(json.dumps(data))



from django.test import TestCase
from django.core.urlresolvers import reverse

from mock import patch, Mock

from tsuru_dashboard.autoscale.context_processors import autoscale_enabled

import os
import urllib


class ContextProcesssorsTest(TestCase):
    def test_autoscale_enabled(self):
        os.environ["AUTOSCALE_DASHBOARD_URL"] = "http://localhost"
        result = autoscale_enabled({})
        self.assertTrue(result["autoscale_enabled"])

    def test_autoscale_not_enabled(self):
        del os.environ["AUTOSCALE_DASHBOARD_URL"]
        result = autoscale_enabled({})
        self.assertFalse(result["autoscale_enabled"])


class IndexTestCase(TestCase):
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    @patch("requests.get")
    def test_index(self, get_mock, token_is_valid):
        response_mock = Mock(status_code=200)
        response_mock.json.return_value = {"name": "appname"}
        get_mock.return_value = response_mock
        token_is_valid.return_value = True

        with self.settings(SESSION_ENGINE='django.contrib.sessions.backends.file'):
            session = self.client.session
            session['tsuru_token'] = "beare token"
            session.save()

            response = self.client.get(reverse("autoscale", args=["app"]))
            self.assertTemplateUsed(response, "autoscale/index.html")

    @patch("tsuru_dashboard.auth.views.token_is_valid")
    @patch("requests.get")
    def test_service_url(self, get_mock, token_is_valid):
        response_mock = Mock(status_code=200)
        response_mock.json.return_value = {"name": "app"}
        get_mock.return_value = response_mock

        token_is_valid.return_value = True

        autoscale_dashboard_url = "http://localhost:123"
        os.environ["AUTOSCALE_DASHBOARD_URL"] = autoscale_dashboard_url
        session_engine = "django.contrib.sessions.backends.file"
        token = "token/+12faslfkl12"

        with self.settings(SESSION_ENGINE=session_engine):
            session = self.client.session
            session['tsuru_token'] = "beare {}".format(token)
            session.save()

            response = self.client.get(reverse("autoscale", args=["app"]))
            expected = "{}/app/{}?TSURU_TOKEN={}".format(
                autoscale_dashboard_url, "app", token)
            self.assertEqual(urllib.unquote(response.context_data["service_url"]), expected)

    @patch("tsuru_dashboard.auth.views.token_is_valid")
    @patch("requests.get")
    def test_app_not_found(self, get_mock, token_is_valid):
        response_mock = Mock(status_code=404)
        get_mock.return_value = response_mock
        token_is_valid.return_value = True

        with self.settings(SESSION_ENGINE='django.contrib.sessions.backends.file'):
            session = self.client.session
            session['tsuru_token'] = "beare token"
            session.save()

            response = self.client.get(reverse("autoscale", args=["app"]))
            self.assertEqual(response.status_code, 404)



from mock import patch, Mock

from django.test import TestCase
from django.test.client import RequestFactory
from django.core.urlresolvers import reverse

from tsuru_dashboard import settings
from tsuru_dashboard.auth.views import LoginRequiredView
from tsuru_dashboard.apps.views import Run
from tsuru_dashboard.apps.forms import RunForm

import json


class RunViewTest(TestCase):
    @patch('requests.get')
    def setUp(self, get):
        self.factory = RequestFactory()
        self.request = self.factory.get('/')
        self.request.session = {}
        content = u'[{"name": "appbla"}, {"name": "appble"}]'
        m = Mock(status_code=200, content=content)
        m.json.return_value = json.loads(content)
        get.return_value = m
        self.response = Run().get(self.request)
        self.request_post = self.factory.post(
            '/',
            {'app': 'app-test', 'command': 'command-test'})
        self.request_post.session = {}
        self.response_mock = Mock()

    def test_should_require_login_to_run_command(self):
        assert issubclass(Run, LoginRequiredView)

    def test_run_should_render_expected_template(self):
        self.assertEqual('apps/run.html', self.response.template_name)

    def test_context_should_contain_form(self):
        self.assertIn('form', self.response.context_data.keys())

    def test_form_in_context_should_has_a_instance_of_RunForm(self):
        form = self.response.context_data.get('form')
        self.assertIsInstance(form, RunForm)

    def test_get_request_run_url_should_return_404(self):
        response = self.client.get(reverse('run'))
        self.assertNotEqual(404, response.status_code)

    @patch('requests.post')
    def test_post_with_app_and_command_sends_post(self, post):
        self.request_post.session = {'tsuru_token': 'tokentest'}
        Run().post(self.request_post)
        self.assertEqual(1, post.call_count)
        post.assert_called_with(
            '%s/apps/app-test/run' % settings.TSURU_HOST,
            data=u'command-test',
            headers={'authorization':
                     self.request_post.session['tsuru_token']})

    @patch('requests.post')
    def test_post_with_valid_app_and_command(self, post):
        content = "command runned"
        post.return_value = Mock(status_code=200, content=content)
        response = Run().post(self.request_post)
        self.assertEqual("command runned",
                         response.context_data.get('message'))
        self.assertIn('form', response.context_data.keys())
        self.assertIsInstance(response.context_data.get('form'), RunForm)

    @patch('requests.post')
    def test_post_with_invalid_app_or_command_error_in_context(self, post):
        post.return_value = Mock(status_code=500, content='Error')
        response = Run().post(self.request_post)
        self.assertEqual('Error', response.context_data.get('errors'))

    @patch('requests.post')
    def test_post_with_invalid_app_or_command(self, post):
        post.return_value = Mock(status_code=500, content='Error')
        response = Run().post(self.request_post)
        self.assertIn('form', response.context_data.keys())
        self.assertIsInstance(response.context_data.get('form'), RunForm)

    @patch('requests.post')
    def test_post_without_app_should_not_send_request_to_tsuru(self, post):
            request = self.factory.post('/', {'app': '',
                                              'command': 'comand-test'})
            request.session = {}
            Run().post(request)
            self.assertEqual(0, post.call_count)

    @patch('requests.get')
    @patch('requests.post')
    def test_post_without_app_should_return_form_with_errors(self, get, post):
        content = (u"""[{"name":"appbla"}, {"name":"appble"}]""")
        m = Mock(status_code=200, content=content)
        m.json.return_value = json.loads(content)
        get.return_value = m
        request = self.factory.post('/', {'app': '',
                                          'command': 'command-test'})
        request.session = {}
        response = Run().post(request)
        self.assertIn('form', response.context_data.keys())
        form = response.context_data.get('form')
        self.assertIsInstance(form, RunForm)
        self.assertEqual(u'This field is required.',
                         form.errors.get('app')[0])



import json
import requests
from tsuruclient import client

from tsuru_dashboard import settings
from django.template.response import TemplateResponse
from django.shortcuts import redirect
from django.views.generic.base import View, TemplateView
from django.views.generic.edit import FormView
from django.core.urlresolvers import reverse
from django.contrib import messages

from .forms import (LoginForm, SignupForm, KeyForm, TokenRequestForm,
                    PasswordRecoveryForm, ChangePasswordForm)


def token_is_valid(token):
    headers = {'authorization': token}
    url = "{0}/users/info".format(settings.TSURU_HOST)
    response = requests.get(url, headers=headers)
    return response.status_code == 200


def get_permissions(token):
    headers = {"authorization": token}
    permissions = {}

    url = "{}/docker/healing".format(settings.TSURU_HOST)
    response = requests.get(url, headers=headers)
    permissions["healing"] = response.status_code != 403
    return permissions


class LoginRequiredMixin(object):

    @property
    def client(self):
        target = settings.TSURU_HOST
        token = self.request.session.get('tsuru_token').split(" ")[-1]
        return client.Client(target, token)

    @property
    def authorization(self):
        return {'authorization': self.request.session.get('tsuru_token')}

    def dispatch(self, request, *args, **kwargs):
        token = request.session.get('tsuru_token')
        if not token or not token_is_valid(token):
            return redirect("%s?next=%s" % (reverse('login'), request.path))
        return super(LoginRequiredMixin, self).dispatch(
            request, *args, **kwargs)


class ChangePassword(LoginRequiredMixin, FormView):
    template_name = 'auth/change_password.html'
    form_class = ChangePasswordForm
    success_url = '/auth/change-password/'

    def form_valid(self, form):
        data = form.cleaned_data
        headers = {'authorization': self.request.session.get('tsuru_token')}
        url = "{0}/users/password".format(settings.TSURU_HOST)
        response = requests.put(url, data=json.dumps(data), headers=headers)
        if response.status_code < 399:
            messages.success(self.request, u'Password successfully updated!', fail_silently=True)
        else:
            messages.error(self.request, response.text, fail_silently=True)
        return super(ChangePassword, self).form_valid(form)


class LoginRequiredView(LoginRequiredMixin, View):
    pass


class TokenRequest(FormView):
    template_name = 'auth/token_request.html'
    form_class = TokenRequestForm
    success_url = '/auth/token-request/success/'

    def form_valid(self, form):
        form.send()
        return super(TokenRequest, self).form_valid(form)


class TokenRequestSuccess(TemplateView):
    template_name = 'auth/token_request_success.html'


class PasswordRecoverySuccess(TemplateView):
    template_name = 'auth/password_recovery_success.html'


class PasswordRecovery(FormView):
    template_name = 'auth/password_recovery.html'
    form_class = PasswordRecoveryForm
    success_url = '/auth/password-recovery/success/'

    def form_valid(self, form):
        form.send()
        return super(PasswordRecovery, self).form_valid(form)


class Login(FormView):
    template_name = 'auth/login.html'
    form_class = LoginForm

    def get_context_data(self, *args, **kwargs):
        self.request.session["next_url"] = self.request.GET.get("next", "/apps")
        data = super(Login, self).get_context_data(*args, **kwargs)
        data["scheme_info"] = scheme = self.scheme_info()
        scheme_data = scheme.get('data', {})
        if scheme_data is None:
            return data
        authorize_url = scheme_data.get('authorizeUrl', '')
        callback_url = "http://{}/auth/callback/".format(self.request.META.get('HTTP_HOST'))
        data["authorize_url"] = authorize_url.replace('__redirect_url__', callback_url)
        return data

    def get_success_url(self):
        return '/apps'

    def scheme_info(self):
        url = '{0}/auth/scheme'.format(settings.TSURU_HOST)
        response = requests.get(url)
        if response.status_code == 200:
            return response.json()
        return {}

    def form_valid(self, form):
        username = form.cleaned_data['username']
        data = {"password": form.cleaned_data['password']}
        url = '{0}/users/{1}/tokens'.format(settings.TSURU_HOST, username)
        response = requests.post(url, data=json.dumps(data))

        if response.status_code == 200:
            result = response.json()
            self.request.session['username'] = username
            self.request.session['tsuru_token'] = "type {0}".format(result['token'])
            self.request.session['permissions'] = get_permissions(result['token'])
            return super(Login, self).form_valid(form)

        form.add_error(None, response.content)
        return self.render_to_response(self.get_context_data(form=form))


class Logout(View):

    def get(self, request):
        if 'tsuru_token' in request.session:
            del request.session['tsuru_token']
        return redirect('/')


class Signup(View):

    def get(self, request):
        return TemplateResponse(request, 'auth/signup.html',
                                context={'signup_form': SignupForm()})

    def post(self, request):
        form = SignupForm(request.POST)
        if not form.is_valid():
            return TemplateResponse(request,
                                    'auth/signup.html',
                                    context={'signup_form': form})
        post_data = {'email': form.data['email'],
                     'password': form.data['password']}
        response = requests.post(
            '%s/users' % settings.TSURU_HOST,
            data=json.dumps(post_data))
        if response.status_code == 201:
            message = 'User "{0}" successfully created!'.format(
                form.data['email']
            )
            return TemplateResponse(request, 'auth/signup.html',
                                    context={'signup_form': SignupForm(),
                                             'message': message})
        else:
            return TemplateResponse(request, 'auth/signup.html',
                                    context={'signup_form': form,
                                             'error': response.content},
                                    status=response.status_code)


class Callback(View):
    def get(self, request):
        code = request.GET.get("code")
        redirect_url = "http://{}/auth/callback/".format(request.META.get('HTTP_HOST'))
        data = {
            "code": code,
            "redirectUrl": redirect_url,
        }
        url = '{}/auth/login'.format(settings.TSURU_HOST)
        response = requests.post(url, data=json.dumps(data))
        if response.status_code == 200:
            result = response.json()
            self.request.session['tsuru_token'] = "type {}".format(result['token'])
            self.request.session['permissions'] = get_permissions(result['token'])
            next_url = self.request.session["next_url"]
            return redirect(next_url)
        return redirect('/auth/login')


class KeyAdd(LoginRequiredMixin, FormView):
    form_class = KeyForm
    template_name = 'auth/key_add.html'
    success_url = '/auth/key/'

    def form_valid(self, form):
        url = '{}/users/keys'.format(settings.TSURU_HOST)
        response = requests.post(url, data=json.dumps(form.cleaned_data), headers=self.authorization)
        if response.status_code < 399:
            messages.success(self.request, "The key was successfully added", fail_silently=True)
        else:
            messages.error(self.request, response.text, fail_silently=True)
        return super(KeyAdd, self).form_valid(form)



from django.conf.urls import patterns, url
from django.views.decorators.csrf import csrf_exempt

import views


urlpatterns = patterns(
    '',
    url(r'^$', views.ListApp.as_view(), name='list-app'),
    url(r'^list.json$', views.ListAppJson.as_view(), name='list-app-json'),
    url(r'^create/$', views.CreateApp.as_view(), name='create-app'),
    url(r'^run/$', views.Run.as_view(), name='run'),
    url(r'^(?P<app_name>[\w-]+).json$',
        views.AppDetailJson.as_view(), name='detail-app-json'),
    url(r'^(?P<app_name>[\w-]+)/$',
        views.AppDetail.as_view(), name='detail-app'),
    url(r'^(?P<name>[\w-]+)/remove/$',
        views.RemoveApp.as_view(), name='remove_app'),
    url(r'^(?P<name>[\w-]+)/unlock/$', views.Unlock.as_view(), name='unlock-app'),
    url(r'^(?P<app_name>[\w-]+)/log/stream/$', views.LogStream.as_view(), name='app-log-stream'),
    url(r'^(?P<app_name>[\w-]+)/log/$', views.AppLog.as_view(),
        name='app_log'),
    url(r'^(?P<app_name>[\w-]+)/settings/$', views.Settings.as_view(),
        name='app-settings'),
    url(r'^(?P<app_name>[\w-]+)/teams/$',
        views.AppTeams.as_view(), name='app-teams'),
    url(r'^(?P<app_name>[\w-]+)/team/add/$',
        views.AppAddTeam.as_view(), name='app-add-team'),
    url(r'^(?P<app_name>[\w-]+)/team/revoke/(?P<team>[\w-]+)',
        views.AppRevokeTeam.as_view(), name='app-revoke-team'),
    url(r'^(?P<app_name>[\w-]+)/units/$',
        views.ChangeUnit.as_view(), name='change-units'),
    url(r'^(?P<app_name>[\w-]+)/metrics/details/$',
        views.MetricDetail.as_view(), name='metrics-app'),
    url(r'^(?P<app_name>[\w-]+)/deploys/$', csrf_exempt(views.ListDeploy.as_view()), name='app-deploys'),
    url(r'^(?P<app_name>[\w-]+)/deploys/(?P<deploy>[\s\w@\.-]+)/$',
        views.DeployInfo.as_view(), name='app-deploy'),
    url(r'^(?P<app_name>[\w-]+)/rollback/(?P<image>.+)/$',
        views.AppRollback.as_view(), name='app-rollback'),
    url(r'^(?P<app_name>[\w-]+)/env/$', views.AppEnv.as_view(), name='get-env'),
)



import json

from mock import patch, Mock

from django.test import TestCase
from django.test.client import RequestFactory
from django.core.urlresolvers import reverse

from tsuru_dashboard import settings
from tsuru_dashboard.apps.views import AppAddTeam
from tsuru_dashboard.auth.views import LoginRequiredView
from tsuru_dashboard.apps.forms import AppAddTeamForm


class AppAddTeamTestCase(TestCase):
    @patch('requests.get')
    def setUp(self, get):
        self.factory = RequestFactory()
        self.request = self.factory.get('/')
        self.request.session = {}
        self.app_name = 'app-test'
        content = u'[{"name": "tsuruteam"}, {"name": "teamvieira"}]'
        m = Mock(status_code=200, content=content)
        m.json.return_value = json.loads(content)
        get.return_value = m
        self.response = AppAddTeam().get(self.request, self.app_name)
        self.request_post = self.factory.post('/', {'team': 'team-test'})
        self.request_post.session = {}
        self.response_mock = Mock()

    def test_should_require_login_to_add_team_to_app(self):
        assert issubclass(AppAddTeam, LoginRequiredView)

    def test_should_render_expected_template(self):
        self.assertIn('apps/app_add_team.html', self.response.template_name)

    def test_context_should_contain_form(self):
        self.assertIn('form', self.response.context_data.keys())

    def test_form_in_context_should_has_a_instance_of_AppAddTeamForm(self):
        form = self.response.context_data.get('form')
        self.assertIsInstance(form, AppAddTeamForm)

    def test_get_request_team_url_should_not_return_404(self):
        response = self.client.get(reverse('app-add-team',
                                   args=[self.app_name]))
        self.assertNotEqual(404, response.status_code)

    @patch('requests.put')
    def test_should_send_request_post_to_tsuru_with_args_expected(self, put):
        self.request_post.session = {'tsuru_token': 'tokentest'}
        AppAddTeam().post(self.request_post, self.app_name)
        self.assertEqual(1, put.call_count)
        put.assert_called_with(
            '%s/apps/app-test/team-test' % settings.TSURU_HOST,
            headers={'authorization':
                     self.request_post.session['tsuru_token']})

    @patch('requests.put')
    def test_post_with_valid_and_team(self, put):
        put.return_value = Mock(status_code=200)
        response = AppAddTeam().post(self.request_post, self.app_name)
        self.assertEqual("The Team was successfully added",
                         response.context_data.get('message'))

    @patch('requests.put')
    def test_post_with_invalid_app_or_team(self, put):
        put.return_value = Mock(content='Error', status_code=500)
        response = AppAddTeam().post(self.request_post, self.app_name)
        self.assertEqual('Error', response.context_data.get('errors'))

    @patch('requests.put')
    def test_post_with_valid_data_should_return_context_with_form(self, put):
        put.return_value = Mock(status_code=200)
        response = AppAddTeam().post(self.request_post, self.app_name)
        self.assertIn('form', response.context_data.keys())
        self.assertIsInstance(response.context_data.get('form'), AppAddTeamForm)

    @patch('requests.put')
    def test_post_with_invalid_data_should_return_context_with_form(self, put):
        put.return_value = Mock(content='Error', status_code=500)
        response = AppAddTeam().post(self.request_post, self.app_name)
        self.assertIn('form', response.context_data.keys())
        self.assertIsInstance(response.context_data.get('form'), AppAddTeamForm)

    @patch('requests.put')
    def test_invalid_post_should_not_send_request_to_tsuru(self, put):
        request = self.factory.post('/', {'team': ''})
        request.session = {}
        AppAddTeam().post(request, self.app_name)
        self.assertEqual(0, put.call_count)

    @patch('requests.put')
    @patch('requests.get')
    def test_post_invalid_form_should_return_form_with_errors(self, get, put):
        content = (u"""[{"name":"teamtsuru"},""" +
                   u"""{"Name":"teamestag"},{"Name":"teambla"}]""")
        m = Mock(status_code=200, content=content)
        m.json.return_value = json.loads(content)
        get.return_value = m
        request = self.factory.post('/', {'team': ''})
        request.session = {}
        response = AppAddTeam().post(request, self.app_name)
        self.assertIn('form', response.context_data.keys())
        form = response.context_data.get('form')
        self.assertIsInstance(form, AppAddTeamForm)
        self.assertEqual(u'This field is required.', form.errors.get('team')[0])



from mock import patch, Mock

from django.test import TestCase
from django.test.client import RequestFactory

from tsuru_dashboard import settings
from tsuru_dashboard.apps.views import ChangeUnit


class ChangeUnitViewTest(TestCase):
    @patch('requests.get')
    @patch('requests.delete')
    def test_remove_unit(self, delete, get):
        data = {
            "name": "app1",
            "units": [
                {"Ip": "10.10.10.10"},
                {"Ip": "9.9.9.9"},
                {"Ip": "8.8.8.8"},
                {"Ip": "7.7.7.7"}
            ],
        }
        response_mock = Mock(status_code=200)
        response_mock.json.return_value = data
        get.return_value = response_mock

        data = {"units": '1'}
        request = RequestFactory().post("/", data)
        request.session = {"tsuru_token": "admin"}

        ChangeUnit.as_view()(request, app_name="app_name")

        delete.assert_called_with(
            '{}/apps/app_name/units'.format(settings.TSURU_HOST),
            data='3',
            headers={'authorization': 'admin'}
        )

        get.assert_called_with(
            '{}/apps/app_name'.format(settings.TSURU_HOST),
            headers={'authorization': 'admin'}
        )

    @patch('requests.get')
    @patch('requests.put')
    def test_add_unit(self, put, get):
        data = {
            "name": "app1",
            "units": [
                {"Ip": "10.10.10.10"},
                {"Ip": "9.9.9.9"},
            ],
        }
        response_mock = Mock(status_code=200)
        response_mock.json.return_value = data
        get.return_value = response_mock

        data = {"units": '10'}
        request = RequestFactory().post("/", data)
        request.session = {"tsuru_token": "admin"}

        ChangeUnit.as_view()(request, app_name="app_name")

        put.assert_called_with(
            '{}/apps/app_name/units'.format(settings.TSURU_HOST),
            data='8',
            headers={'authorization': 'admin'}
        )

        get.assert_called_with(
            '{}/apps/app_name'.format(settings.TSURU_HOST),
            headers={'authorization': 'admin'}
        )

    @patch('requests.get')
    def test_redirect_to_the_app_detail_page(self, get):
        data = {
            "name": "app1",
            "units": [
                {"Ip": "10.10.10.10"},
                {"Ip": "9.9.9.9"},
            ],
        }
        response_mock = Mock(status_code=200)
        response_mock.json.return_value = data
        get.return_value = response_mock

        data = {"units": '2'}
        request = RequestFactory().post("/", data)
        request.session = {"tsuru_token": "admin"}

        response = ChangeUnit.as_view()(request, app_name="app_name")

        self.assertEqual(302, response.status_code)
        self.assertEqual("/apps/app_name/", response.items()[1][1])

        get.assert_called_with(
            '{}/apps/app_name'.format(settings.TSURU_HOST),
            headers={'authorization': 'admin'}
        )



from mock import patch, Mock

from django.test import TestCase
from django.test.client import RequestFactory
from django.core.urlresolvers import reverse

from tsuru_dashboard import settings
from tsuru_dashboard.apps.views import AppTeams


class AppTeamsViewTest(TestCase):

    @patch('requests.get')
    def setUp(self, get):
        self.factory = RequestFactory()
        self.request = self.factory.get('/')
        self.request.session = {'tsuru_token': 'tokentest'}
        self.app_name = 'app-teste'
        self.response_mock = Mock()
        self.response_mock.status_code = 200
        self.response_mock.content = '{}'
        get.return_value = self.response_mock
        self.response = AppTeams.as_view()(self.request, app_name=self.app_name)

    def test_run_should_render_expected_template(self):
        self.assertIn('apps/app_team.html', self.response.template_name)

    def test_context_should_contain_app(self):
        self.assertIn('app', self.response.context_data.keys())

    @patch('requests.get')
    @patch('tsuru_dashboard.auth.views.token_is_valid')
    def test_get_sends_request_to_tsuru_with_args_expected(self, token_is_valid, get):
        AppTeams.as_view()(self.request, app_name=self.app_name)
        url = "{}/apps/{}".format(settings.TSURU_HOST, self.app_name)
        get.assert_called_with(
            url, headers={'authorization': self.request.session['tsuru_token']})

    @patch("requests.get")
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_get_with_valid_app_should_return_expected_context(self, tis, get):
        tis.return_value = True
        expected = {
            "Name": "app1",
            "Framework": "php",
            "Repository": "git@git.com:php.git",
            "State": "dead",
            "Units": [
                {"Ip": "10.10.10.10"},
                {"Ip": "9.9.9.9"}
            ],
            "Teams": ["tsuruteam", "crane"]
        }
        response_mock = Mock(status_code=200)
        response_mock.json.return_value = expected
        get.return_value = response_mock

        response = AppTeams.as_view()(self.request, app_name="app1")

        self.assertDictEqual(expected, response.context_data["app"])

    @patch('requests.get')
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_get_with_invalid_app_should_return_context_with_error(self, tis, get):
        self.response_mock.status_code = 404
        self.response_mock.content = 'App not found'
        get.return_value = self.response_mock

        response = AppTeams.as_view()(self.request, app_name='invalid-app')

        self.assertIn('errors', response.context_data.keys())
        self.assertEqual(self.response_mock.content, response.context_data['errors'])

    def test_get_request_run_url_should_not_return_404(self):
        response = self.client.get(reverse('app-teams', args=[self.app_name]))
        self.assertNotEqual(404, response.status_code)



from django.test import TestCase
from django.forms import CharField

from tsuru_dashboard.apps import forms


class AppAddTeamFormTest(TestCase):
    def test_forms_should_contains_AppAddTeamForm(self):
        self.assertTrue(hasattr(forms, 'AppAddTeamForm'))

    def test_should_have_team_field(self):
        self.assertIn('team', forms.AppAddTeamForm.base_fields)

    def test_team_field_should_have_CharField(self):
        field = forms.AppAddTeamForm.base_fields['team']
        self.assertIsInstance(field, CharField)

    def test_team_field_should_have_at_most_60_characters(self):
        field = forms.AppAddTeamForm.base_fields['team']
        self.assertEqual(60, field.max_length)



from django.http import HttpResponse

from tsuru_dashboard import settings

import requests


def healthcheck(request):
    url = "{0}/healthcheck/".format(settings.TSURU_HOST)

    try:
        response = requests.get(url)
    except requests.exceptions.ConnectionError:
        return HttpResponse("Failed to connect to tsuru.", status=500)

    if response.status_code != 200 or response.text != "WORKING":
        return HttpResponse("Failed to connect to tsuru.", status=500)

    return HttpResponse("WORKING", status=200)



from django.test import TestCase
from django.test.client import RequestFactory
from django.core.urlresolvers import reverse

from tsuru_dashboard import settings
from tsuru_dashboard.services.views import Unbind

from mock import patch, Mock


class UnbindViewTest(TestCase):
    @patch("requests.delete")
    @patch("requests.get")
    def test_view(self, get, delete):
        get.return_value = Mock(status_code=200)
        request = RequestFactory().get("/")
        request.session = {"tsuru_token": "admin"}
        instance = "instance"
        app = "app"
        service = "service"

        response = Unbind.as_view()(request, service=service, instance=instance, app=app)

        self.assertEqual(302, response.status_code)
        url = reverse('service-detail', args=[service, instance])
        self.assertEqual(url, response.items()[1][1])
        url = '{}/services/{}/instances/{}/{}'.format(settings.TSURU_HOST, service, instance, app)
        delete.assert_called_with(url, headers={'authorization': 'admin'})



from mock import patch

from django.test import TestCase
from django.test.client import RequestFactory
from django.core.urlresolvers import reverse

from tsuru_dashboard import settings
from tsuru_dashboard.apps.views import AppRevokeTeam


class AppRevokeTeamTestCase(TestCase):

    @patch("requests.delete")
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_view(self, token_is_valid, delete):
        token_is_valid.return_value = True
        request = RequestFactory().get("/")
        request.session = {"tsuru_token": "admin"}
        app_name = "app_name"
        team = "team"
        response = AppRevokeTeam.as_view()(request,
                                           app_name=app_name,
                                           team=team)
        self.assertEqual(302, response.status_code)
        url = reverse('app-teams', args=[app_name])
        self.assertEqual(url, response.items()[1][1])
        delete.assert_called_with(
            "{0}/apps/{1}/{2}".format(
                settings.TSURU_HOST,
                app_name, team
            ),
            headers={'authorization': 'admin'})



# coding: utf-8
from mock import patch, Mock

from django.test import TestCase
from django.test.client import RequestFactory
from django.core.urlresolvers import reverse

from tsuru_dashboard import settings
from tsuru_dashboard.auth.views import LoginRequiredView
from tsuru_dashboard.apps.views import AppEnv
from tsuru_dashboard.apps.forms import SetEnvForm


class AppEnvViewTest(TestCase):

    @patch('requests.get')
    def setUp(self, get):
        self.factory = RequestFactory()
        self.request = self.factory.get('/')
        self.request.session = {'tsuru_token': 'tokentest'}
        self.request_post = self.factory.post('/', {'env': 'env-test'})
        self.request_post.session = {}
        self.app_name = 'app-teste'
        self.view = AppEnv()
        env_mock = Mock(status_code=200, content='env1\nenv2\n')
        self.view.get_envs = Mock(return_value=env_mock)
        get.return_value = env_mock
        self.response = AppEnv().get(self.request, self.app_name)

    def test_should_require_login_to_set_env(self):
        assert issubclass(AppEnv, LoginRequiredView)

    def test_run_should_render_expected_template(self):
        self.assertEqual('apps/app_env.html', self.response.template_name)

    def test_context_should_contain_envs(self):
        self.assertIn('envs', self.response.context_data.keys())

    def test_context_should_contain_app(self):
        self.assertIn('app', self.response.context_data.keys())

    def test_app_on_context_should_contain_app_name(self):
        self.assertEqual(self.app_name, self.response.context_data['app'])

    @patch('requests.get')
    def test_get_with_app_should_send_request_to_tsuru(self, get):
        AppEnv().get(self.request, self.app_name)
        get.assert_called_with(
            '%s/apps/%s/env' % (settings.TSURU_HOST, self.app_name),
            headers={'authorization': self.request.session['tsuru_token']})

    @patch('requests.get')
    def test_get_with_valid_app_should_return_expected_context(self, get):
        content = "env1\nenv2\n"
        get.return_value = Mock(status_code=200, content=content)
        response = AppEnv().get(self.request, self.app_name)
        expected_response = content.split('\n')
        self.assertEqual(expected_response, response.context_data['envs'])

    @patch('requests.get')
    def test_get_with_invalid_app_should_return_context_with_error(self, get):
        content = 'App not found'
        get.return_value = Mock(status_code=404, content=content)
        response = AppEnv().get(self.request, 'invalid-app')
        self.assertIn('errors', response.context_data.keys())
        self.assertEqual(content, response.context_data['errors'])

    def test_get_request_to_url_should_not_return_404(self):
        response = self.client.get(reverse('get-env', args=[self.app_name]))
        self.assertNotEqual(404, response.status_code)

    def test_context_should_contain_form(self):
        self.assertIn('form', self.response.context_data.keys())

    def test_form_in_context_should_has_a_instance_of_SetEnvForm(self):
        form = self.response.context_data.get('form')
        self.assertIsInstance(form, SetEnvForm)

    def test_form_in_context_should_has_initial_app_name(self):
        form = self.response.context_data.get('form')
        self.assertTrue({'app': self.app_name}, form.initial)

    @patch('requests.post')
    def test_post_should_send_request_post_to_tsuru(self, post):
        self.request_post.session = {'tsuru_token': 'tokentest'}
        self.view.post(self.request_post, self.app_name)
        self.assertEqual(1, post.call_count)
        post.assert_called_with(
            u'%s/apps/%s/env' % (settings.TSURU_HOST, self.app_name),
            data=u'env-test',
            headers={'authorization':
                     self.request_post.session['tsuru_token']})

    @patch('requests.post')
    def test_postshould_return_context_with_message_expected(self, post):
        post.return_value = Mock(content="env added", status_code=200)
        response = self.view.post(self.request_post, self.app_name)
        self.assertEqual("env added", response.context_data.get('message'))

    @patch('requests.post')
    def test_invalid_post_should_return_error_message(self, post):
        post.return_value = Mock(content='Error', status_code=500)
        response = self.view.post(self.request_post, 'invalid-app')
        self.assertEqual('Error', response.context_data.get('errors'))

    @patch('requests.post')
    def test_valid_post_returns_context_with_form(self, post):
        post.return_value = Mock(status_code=200)
        response = self.view.post(self.request_post, self.app_name)
        self.assertIn('form', response.context_data.keys())
        self.assertIsInstance(response.context_data.get('form'), SetEnvForm)

    @patch('requests.post')
    def test_post_without_env_should_return_form_with_errors(self, post):
        request = self.factory.post('/', {'env': ''})
        request.session = {}
        response = self.view.post(request, self.app_name)
        self.assertIn('form', response.context_data.keys())
        form = response.context_data.get('form')
        self.assertIsInstance(form, SetEnvForm)
        self.assertEqual(u'This field is required.',
                         form.errors.get('env')[0])

    @patch('requests.post')
    def test_post_with_valid_app_returns_env_list(self, post):
        content = 'env1\nenv2\n'
        post.return_value = Mock(content=content)
        response = self.view.post(self.request_post, self.app_name)
        expected_response = content.split('\n')
        expected_response.append(self.request_post.POST['env'])
        self.assertIn('envs', response.context_data.keys())
        self.assertEqual(expected_response, response.context_data['envs'])

    def test_post_request_to_url_should_not_return_404(self):
        response = self.client.post(reverse('get-env', args=[self.app_name]),
                                    self.request_post.POST)
        self.assertNotEqual(404, response.status_code)



from mock import patch

from django.test import TestCase
from django.test.client import RequestFactory

import httpretty
import json

from tsuru_dashboard import settings
from tsuru_dashboard.admin.views import TemplateListJson


class TemplateListJsonTest(TestCase):
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def setUp(self, token_is_valid):
        token_is_valid.return_value = True
        httpretty.enable()

        self.factory = RequestFactory()
        self.request = self.factory.get('/')
        self.request.session = {'tsuru_token': 'tokentest'}

        url = "{}/iaas/templates".format(settings.TSURU_HOST)
        self.templates = []
        httpretty.register_uri(
            httpretty.GET,
            url,
            body=json.dumps(self.templates),
            status=200
        )
        self.response = TemplateListJson.as_view()(self.request)

    def tearDown(self):
        httpretty.disable()
        httpretty.reset()

    def teste_view(self):
        data = json.loads(self.response.content)
        self.assertListEqual(self.templates, data)



from django.conf.urls import patterns, url
from django.views.decorators.cache import cache_page

import views

urlpatterns = patterns(
    '',
    url(r'^$', views.DashboardView.as_view(), name='dashboard'),
    url(r'^healing_status$', cache_page(60 * 5)(views.HealingView.as_view()), name='healing'),
    url(r'^cloud_status$', cache_page(60 * 30)(views.CloudStatusView.as_view()), name='cloud'),
    url(r'^deploys$', cache_page(60 * 5)(views.DeploysView.as_view()), name='deploys'),
)



import copy
import json

from django.test import TestCase
from django.test.client import RequestFactory

from tsuru_dashboard import settings
from tsuru_dashboard.apps.views import DeployInfo

from mock import patch, Mock

import httpretty


class InfoViewTest(TestCase):

    def setUp(self):
        self.request = RequestFactory().get("/")
        self.request.session = {"tsuru_token": "admin"}
        self.data = {u'Id': u'53e143cb874ccb1f68000001',
                     u'App': u'g1',
                     u'Timestamp': u'18-08-2014 11:29:32',
                     u'Duration': u'00m23s',
                     u'Commit': u'e82nn93nd93mm12o2ueh83dhbd3iu112',
                     u'Error': u'',
                     u'Diff': u'test_diff'}

    @httpretty.activate
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_view(self, token_is_valid):
        token_is_valid.return_value = True

        url = '{}/deploys/{}'.format(settings.TSURU_HOST, "53e143cb874ccb1f68000001")
        body = json.dumps(self.data)
        httpretty.register_uri(httpretty.GET, url, body=body)

        url = '{}/apps/{}'.format(settings.TSURU_HOST, "app_name")
        body = json.dumps({"name": "app_name"})
        httpretty.register_uri(httpretty.GET, url, body=body)

        view = DeployInfo
        response = view.as_view()(
            self.request,
            deploy="53e143cb874ccb1f68000001",
            app_name="app_name",
        )

        expected = copy.deepcopy(self.data)
        expected["Diff"] = u"""<div class="highlight"><pre>%s\n</pre></div>\n""" % self.data["Diff"]
        self.assertEqual("apps/deploy.html", response.template_name[0])
        self.assertDictEqual(expected, response.context_data['deploy'])
        self.assertIn('app', response.context_data.keys())

    @patch("requests.get")
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_view_without_diff(self, token_is_valid, get):
        token_is_valid.return_value = True
        data = copy.deepcopy(self.data)
        del data["Diff"]
        response_mock = Mock()
        response_mock.json.return_value = copy.deepcopy(data)
        get.return_value = response_mock

        view = DeployInfo
        view.get_app = Mock()
        response = view.as_view()(
            self.request,
            deploy="53e143cb874ccb1f68000001",
            app_name="app_name",
        )

        data["Diff"] = None

        self.assertEqual("apps/deploy.html", response.template_name[0])
        self.assertDictEqual(data, response.context_data['deploy'])
        self.assertIn('app', response.context_data.keys())
        url = '{0}/deploys/{1}'.format(settings.TSURU_HOST, "53e143cb874ccb1f68000001")
        get.assert_called_with(url, headers={'authorization': 'admin'})



from dateutil import parser
from django import template

import time

register = template.Library()


@register.filter
def string_to_date(value):
    deploy_date = parser.parse(value)
    return deploy_date


@register.filter
def time_to_string(value):
    return time.strftime('%Mm%Ss', time.gmtime(float(value)/1000000000))



from django.views.generic import TemplateView
from django.http import Http404

from tsuru_dashboard import settings
from tsuru_dashboard.auth.views import LoginRequiredMixin

import os
import urllib

import requests


class Index(LoginRequiredMixin, TemplateView):
    template_name = 'autoscale/index.html'

    def get_app(self, app_name):
        url = '{}/apps/{}'.format(settings.TSURU_HOST, app_name)
        response = requests.get(url, headers=self.authorization)

        if response.status_code != 200:
            raise Http404()

        return response.json()

    def get_context_data(self, *args, **kwargs):
        context = super(Index, self).get_context_data(*args, **kwargs)
        token = self.request.session.get('tsuru_token').split(' ')[1]
        token = urllib.quote_plus(token)

        app = self.get_app(kwargs["app"])

        service_url = "{}/app/{}?TSURU_TOKEN={}".format(
            os.environ.get("AUTOSCALE_DASHBOARD_URL"), app["name"], token)

        context["service_url"] = service_url
        context["app"] = app
        return context



from django.test import TestCase
from django.forms import CharField

from tsuru_dashboard.apps import forms


class RunFormTest(TestCase):
    def test_forms_should_contains_RunForm(self):
        self.assertTrue(hasattr(forms, 'RunForm'))

    def test_should_have_app_field(self):
        self.assertIn('app', forms.RunForm.base_fields)

    def test_app_field_should_have_CharField(self):
        field = forms.RunForm.base_fields['app']
        self.assertIsInstance(field, CharField)

    def test_app_field_should_have_at_most_60_characters(self):
        field = forms.RunForm.base_fields['app']
        self.assertEqual(60, field.max_length)

    def test_should_have_command_field(self):
        self.assertIn('command', forms.RunForm.base_fields)

    def test_command_field_should_have_CharField(self):
        field = forms.RunForm.base_fields['command']
        self.assertIsInstance(field, CharField)

    def test_command_field_should_have_at_most_1000_characters(self):
        field = forms.RunForm.base_fields['command']
        self.assertEqual(1000, field.max_length)



from django.conf.urls import patterns, url

import views


urlpatterns = patterns(
    '',
    url(r'^login/$', views.Login.as_view(), name='login'),
    url(r'^logout/$', views.Logout.as_view(), name='logout'),
    url(r'^key/$', views.KeyAdd.as_view(), name='key'),
    url(r'^signup/$', views.Signup.as_view(), name='signup'),
    url(r'^callback/$', views.Callback.as_view(), name='callback'),
    url(r'^token-request/$', views.TokenRequest.as_view(), name='token-request'),
    url(r'^token-request/success/$', views.TokenRequestSuccess.as_view(),
        name='token-request-success'),
    url(r'^password-recovery/$', views.PasswordRecovery.as_view(), name='password-recovery'),
    url(r'^password-recovery/success/$', views.PasswordRecoverySuccess.as_view(),
        name='password-recovery-success'),
    url(r'^change-password/$', views.ChangePassword.as_view(), name='change-password'),
)
